<?php

namespace Core\Router;

use Core\Exceptions\Exceptions;
use Core\Http\Request;
use Core\View\Render;
use Exception;

class Router
{
    /**
     * List of routes
     *
     * @var array
     */
    protected static array $routes = [];

    /**
     * Add GET route
     *
     * @param string $route
     * @param string $controller
     * @param string $action
     * @return void
     */
    public static function get(string $route, string $controller, string $action): void
    {
        self::register($route, $controller, $action, "GET");
    }

    /**
     * Add POST route
     *
     * @param string $route
     * @param string $controller
     * @param string $action
     * @return void
     */
    public static function post(string $route, string $controller, string $action): void
    {
        self::register($route, $controller, $action, "POST");
    }

    /**
     * Register route
     *
     * @param string $route
     * @param string $controller
     * @param string $action
     * @param string $method
     * @return void
     */
    public static function register(string $route, string $controller, string $action, string $method): void
    {
        self::$routes[$method][$route] = [
            'controller' => $controller,
            'action'     => $action,
        ];
    }

    /**
     * Dispatch router and run application
     *
     * @return void
     */
    public static function dispatch(): void
    {
        // Init request
        $request = new Request();
        $url     = $request->url();
        $method  = $request->method();

        // Capture all exceptions
        Exceptions::catch($request);

        try {
            if (array_key_exists($url, self::$routes[$method])) {
                $controllerName = self::$routes[$method][$url]['controller'];
                $actionName     = self::$routes[$method][$url]['action'];

                // Check for controller existence
                if (!class_exists($controllerName)) {
                    throw new Exception(sprintf("Controller '%s' missing", $controllerName));
                }

                // Create controller object
                $controller = new $controllerName($request);

                // Check for method on controller
                if (!method_exists($controller, $actionName)) {
                    throw new Exception(sprintf("Method '%s' not found on '%s'", $actionName, $controllerName));
                }

                // Build response object of action
                $response = $controller->$actionName();

                // Render action
                if ($response instanceof Render) {
                    $response->render();
                }
            } else {
                throw new Exception(sprintf("No route found for: %s", $url));
            }
        } catch (Exception $e) {
            Exceptions::catchOne($e);
        }
    }
}